import chalk from 'chalk'
import Axios, { AxiosRequestConfig, AxiosResponse } from 'axios'

const DEF_TIMEOUT = 10000
const DEF_RETRIES = 2
const DEF_UA = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.80 Safari/537.36'

export const axios = Axios.create({
  headers: {
    'user-agent': DEF_UA
  },
  withCredentials: true
})

export interface RequestConfig extends AxiosRequestConfig {
  timeout?: number
  maxRetries?: number
}

export const request = (config: RequestConfig, retries = 0): Promise<AxiosResponse<any>> => {
  config.timeout = config.timeout || DEF_TIMEOUT
  config.maxRetries = config.maxRetries || DEF_RETRIES
  const _config = JSON.stringify(config)
  const cancelTokenSource = Axios.CancelToken.source()
  config.cancelToken = cancelTokenSource.token

  console.debug(chalk.white(`[request] ${config.method || 'GET'} ${config.url}`, config.params ? JSON.stringify(config.params) : ''))

  return new Promise((resolve, reject) => {
    let done = false

    const timer = setTimeout(() => {
      if (!done) {
        cancelTokenSource.cancel()

        // @ts-ignore
        if (retries >= config.maxRetries) {
          reject(new Error(`max retries exceeded (${retries})`))
        } else {
          retries++
          console.info(chalk.magenta(`[request] ${config.url} timeouted, canceled, retry ${retries}`))
          resolve(request(JSON.parse(_config), retries))
        }
      }
    }, config.timeout)

    const startAt = Date.now()

    axios(config)
      .then((res) => {
        console.debug(chalk.white(`└─ ${res.status} ${Date.now() - startAt}ms`))
        resolve(res)
      })
      .catch((e) => {
        if (e.response && e.response.data) {
          console.error('[error.message]', e.message)
          console.error('[error.response]', e.response.data)
        }
        reject(e)
      })
      .finally(() => {
        done = true
        clearTimeout(timer)
      })
  })
}
